@@ -145,6 +145,7 @@ meta_shaped_texture_dispose (GObject *object)
   priv->paint_tower = NULL;
 
   g_clear_pointer (&priv->texture, cogl_object_unref);
+  g_clear_pointer (&priv->opaque_region, cairo_region_destroy);
 
   meta_shaped_texture_set_mask_texture (self, NULL);
   meta_shaped_texture_set_clip_region (self, NULL);
@@ -236,7 +237,7 @@ set_cogl_texture (MetaShapedTexture *stex,
   if (priv->texture)
     cogl_object_unref (priv->texture);
 
-  priv->texture = cogl_tex;
+  priv->texture = cogl_object_ref (cogl_tex);
 
   if (cogl_tex != NULL)
     {
@@ -690,7 +691,7 @@ wayland_surface_update_area (MetaShapedTexture *stex,
     }
 }
 
-static void
+static gboolean
 get_clip (MetaShapedTexture *stex,
           int x,
           int y,
@@ -716,15 +717,12 @@ get_clip (MetaShapedTexture *stex,
    * it here.
    */
   if (!clutter_actor_has_allocation (self))
-    {
-      clutter_actor_queue_redraw (self);
-      return;
-    }
+    return FALSE;
 
   priv = stex->priv;
 
   if (priv->tex_width == 0 || priv->tex_height == 0)
-    return;
+    return FALSE;
 
   clutter_actor_get_allocation_box (self, &allocation);
 
@@ -735,6 +733,8 @@ get_clip (MetaShapedTexture *stex,
   clip->y = y * scale_y;
   clip->width = width * scale_x;
   clip->height = height * scale_y;
+
+  return TRUE;
 }
 
 /**
@@ -765,6 +765,7 @@ meta_shaped_texture_update_area (MetaShapedTexture *stex,
 {
   MetaShapedTexturePrivate *priv;
   cairo_rectangle_int_t clip;
+  gboolean has_clip;
 
   priv = stex->priv;
 
@@ -784,7 +785,7 @@ meta_shaped_texture_update_area (MetaShapedTexture *stex,
 
   meta_texture_tower_update_area (priv->paint_tower, x, y, width, height);
 
-  get_clip (stex, x, y, width, height, &clip);
+  has_clip = get_clip (stex, x, y, width, height, &clip);
 
   if (unobscured_region)
     {
@@ -794,7 +795,8 @@ meta_shaped_texture_update_area (MetaShapedTexture *stex,
         return FALSE;
 
       intersection = cairo_region_copy (unobscured_region);
-      cairo_region_intersect_rectangle (intersection, &clip);
+      if (has_clip)
+	cairo_region_intersect_rectangle (intersection, &clip);
 
       if (!cairo_region_is_empty (intersection))
         {
@@ -811,7 +813,10 @@ meta_shaped_texture_update_area (MetaShapedTexture *stex,
       return FALSE;
     }
 
-  clutter_actor_queue_redraw_with_clip (CLUTTER_ACTOR (stex), &clip);
+  if (has_clip)
+    clutter_actor_queue_redraw_with_clip (CLUTTER_ACTOR (stex), &clip);
+  else
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (stex));
 
   return TRUE;
 }
@@ -870,26 +875,7 @@ meta_shaped_texture_attach_wayland_buffer (MetaShapedTexture  *stex,
   g_return_if_fail (priv->wayland.surface->buffer_ref.buffer == buffer);
 
   if (buffer)
-    {
-      CoglContext *ctx =
-        clutter_backend_get_cogl_context (clutter_get_default_backend ());
-      CoglError *catch_error = NULL;
-      CoglTexture *texture =
-        COGL_TEXTURE (cogl_wayland_texture_2d_new_from_buffer (ctx,
-                                                               buffer->resource,
-                                                               &catch_error));
-      if (!texture)
-        {
-          cogl_error_free (catch_error);
-        }
-      else
-        {
-          buffer->width = cogl_texture_get_width (texture);
-          buffer->height = cogl_texture_get_height (texture);
-        }
-
-      set_cogl_texture (stex, texture);
-    }
+    set_cogl_texture (stex, buffer->texture);
   else
     set_cogl_texture (stex, NULL);
 
