@@ -215,6 +215,14 @@ meta_cursor_get_type (void)
         { META_CURSOR_NW_RESIZE, "META_CURSOR_NW_RESIZE", "nw-resize" },
         { META_CURSOR_MOVE_OR_RESIZE_WINDOW, "META_CURSOR_MOVE_OR_RESIZE_WINDOW", "move-or-resize-window" },
         { META_CURSOR_BUSY, "META_CURSOR_BUSY", "busy" },
+        { META_CURSOR_DND_IN_DRAG, "META_CURSOR_DND_IN_DRAG", "dnd-in-drag" },
+        { META_CURSOR_DND_MOVE, "META_CURSOR_DND_MOVE", "dnd-move" },
+        { META_CURSOR_DND_COPY, "META_CURSOR_DND_COPY", "dnd-copy" },
+        { META_CURSOR_DND_UNSUPPORTED_TARGET, "META_CURSOR_DND_UNSUPPORTED_TARGET", "dnd-unsupported-target" },
+        { META_CURSOR_POINTING_HAND, "META_CURSOR_POINTING_HAND", "pointing-hand" },
+        { META_CURSOR_CROSSHAIR, "META_CURSOR_CROSSHAIR", "crosshair" },
+        { META_CURSOR_IBEAM, "META_CURSOR_IBEAM", "ibeam" },
+        { META_CURSOR_LAST, "META_CURSOR_LAST", "last" },
         { 0, NULL, NULL }
       };
       GType g_enum_type_id;
